=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-09-15 02:14:38 +0000
+++ bzrlib/tests/__init__.py	2006-09-15 05:19:28 +0000
@@ -942,7 +942,7 @@
             # make sure the code path is used, and we don't break on win32
             cleanup_environment()
             process = Popen([sys.executable, bzr_path] + list(process_args),
-                             stdout=PIPE, stderr=PIPE)
+                             stdin=PIPE, stdout=PIPE, stderr=PIPE)
         finally:
             restore_environment()
         return process

=== modified file 'bzrlib/tests/blackbox/test_serve.py'
--- bzrlib/tests/blackbox/test_serve.py	2006-09-13 06:53:25 +0000
+++ bzrlib/tests/blackbox/test_serve.py	2006-09-15 05:19:28 +0000
@@ -20,17 +20,68 @@
 import signal
 
 from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir
 from bzrlib.tests import TestCaseWithTransport
+from bzrlib.transport import smart
+
+
+class DoesNotCloseStdOutClient(smart.SmartStreamClient):
+    """A client that doesn't close stdout upon disconnect().
+    
+    We wish to let stdout remain open so that we can see if the server writes
+    anything to stdout during its shutdown.
+    """
+
+    def disconnect(self):
+        if self._connected:
+            self._connected = False
+            # The client's out is the server's in.
+            self._out.close()
 
 
 class TestBzrServe(TestCaseWithTransport):
+
+    def test_bzr_serve_inet(self):
+        # Make a branch
+        self.make_branch('.')
+
+        # Serve that branch from the current directory
+        process = self.start_bzr_subprocess(['serve', '--inet'])
+
+        # Connect to the server
+        # We use this url because while this is no valid URL to connect to this
+        # server instance, the transport needs a URL.
+        client = DoesNotCloseStdOutClient(
+            lambda: (process.stdout, process.stdin))
+        transport = smart.SmartTransport('bzr://localhost/', client=client)
+
+        # We get a working branch
+        branch = BzrDir.open_from_transport(transport).open_branch()
+        branch.repository.get_revision_graph()
+        self.assertEqual(None, branch.last_revision())
+
+        # finish with the transport
+        del transport
+        # Disconnect the client forcefully JUST IN CASE because of __del__'s use
+        # in the smart module.
+        client.disconnect()
+
+        # Shutdown the server: the client should have disconnected cleanly and
+        # closed stdin, so the server process should shut itself down.
+        self.assertTrue(process.stdin.closed)
+        # Hide stdin from the subprocess module, so it won't fail to close it.
+        process.stdin = None
+        result = self.finish_bzr_subprocess(process, retcode=0)
+        self.assertEqual('', result[0])
+        self.assertEqual('', result[1])
     
     def test_bzr_serve_port(self):
         # Make a branch
         self.make_branch('.')
 
         # Serve that branch from the current directory
-        process = self.start_bzr_subprocess('serve', '--port', 'localhost:0')
+        process = self.start_bzr_subprocess(['serve', '--port', 'localhost:0'],
+                                            skip_if_plan_to_signal=True)
         port_line = process.stdout.readline()
         prefix = 'listening on port: '
         self.assertStartsWith(port_line, prefix)

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2006-09-13 03:53:13 +0000
+++ bzrlib/tests/test_smart_transport.py	2006-09-15 05:19:28 +0000
@@ -22,13 +22,17 @@
 import sys
 
 import bzrlib
-from bzrlib import tests, errors, bzrdir
-from bzrlib.transport import local, memory, smart, get_transport
-
-## class SmartURLTests(tests.TestCase):
-##     """Tests for handling of URLs and detection of smart servers"""
-## 
-##     def test_bzr_url_is_smart(self):
+from bzrlib import (
+        bzrdir,
+        errors,
+        tests,
+        )
+from bzrlib.transport import (
+        get_transport,
+        local,
+        memory,
+        smart,
+        )
 
 
 class SmartClientTests(tests.TestCase):
@@ -75,7 +79,7 @@
 
     def test_canned_get_response(self):
         transport = memory.MemoryTransport('memory:///')
-        transport.put('testfile', StringIO('contents\nof\nfile\n'))
+        transport.put_bytes('testfile', 'contents\nof\nfile\n')
         to_server = StringIO('get\001./testfile\n')
         from_server = StringIO()
         server = smart.SmartStreamServer(to_server, from_server, transport)
@@ -148,13 +152,13 @@
 
     def test_smart_transport_has(self):
         """Checking for file existence over smart."""
-        self.backing_transport.put("foo", StringIO("contents of foo\n"))
+        self.backing_transport.put_bytes("foo", "contents of foo\n")
         self.assertTrue(self.transport.has("foo"))
         self.assertFalse(self.transport.has("non-foo"))
 
     def test_smart_transport_get(self):
         """Read back a file over smart."""
-        self.backing_transport.put("foo", StringIO("contents\nof\nfoo\n"))
+        self.backing_transport.put_bytes("foo", "contents\nof\nfoo\n")
         fp = self.transport.get("foo")
         self.assertEqual('contents\nof\nfoo\n', fp.read())
         
@@ -238,6 +242,38 @@
         self.assertEqual('example.com', t._host)
 
 
+class FakeClient(smart.SmartStreamClient):
+    """Emulate a client for testing a transport's use of the client."""
+
+    def __init__(self):
+        smart.SmartStreamClient.__init__(self, None)
+        self._calls = []
+
+    def _call(self, *args):
+        self._calls.append(('_call', args))
+        return ('ok', )
+
+    def _recv_bulk(self):
+        return 'bar'
+
+
+class TestSmartTransport(tests.TestCase):
+        
+    def test_use_connection_factory(self):
+        # We want to be able to pass a client as a parameter to SmartTransport.
+        client = FakeClient()
+        transport = smart.SmartTransport('bzr://localhost/', client=client)
+
+        # We want to make sure the client is used when the first remote
+        # method is called.  No method should have been called yet.
+        self.assertEqual([], client._calls)
+
+        # Now call a method that should result in a single request.
+        self.assertEqual('bar', transport.get_bytes('foo'))
+        # The only call to _call should have been to get /foo.
+        self.assertEqual([('_call', ('get', '/foo'))], client._calls)
+
+
 # TODO: Client feature that does get_bundle and then installs that into a
 # branch; this can be used in place of the regular pull/fetch operation when
 # coming from a smart server.

=== modified file 'bzrlib/transport/smart.py'
--- bzrlib/transport/smart.py	2006-09-13 06:45:03 +0000
+++ bzrlib/transport/smart.py	2006-09-15 05:19:28 +0000
@@ -172,11 +172,12 @@
         return None
     if req_line[-1] != '\n':
         raise BzrProtocolError("request %r not terminated" % req_line)
-    return tuple((a.decode('utf-8') for a in req_line[:-1].split('\1')))
+    return tuple((a.decode('utf-8') for a in req_line[:-1].split('\x01')))
 
 
 def _send_tuple(to_file, args):
-    to_file.write('\1'.join((a.encode('utf-8') for a in args)) + '\n')
+    # XXX: this will be inefficient.  Just ask Robert.
+    to_file.write('\x01'.join((a.encode('utf-8') for a in args)) + '\n')
     to_file.flush()
 
 
@@ -528,13 +529,20 @@
     type: SmartTCPTransport, etc.
     """
 
-    def __init__(self, server_url, clone_from=None):
+    def __init__(self, server_url, clone_from=None, client=None):
+        """Constructor.
+
+        :param client: ignored when clone_from is not None.
+        """
         ### Technically super() here is faulty because Transport's __init__
         ### fails to take 2 parameters, and if super were to choose a silly
         ### initialisation order things would blow up. 
         super(SmartTransport, self).__init__(server_url)
         if clone_from is None:
-            self._client = SmartStreamClient(self._connect_to_server)
+            if client is None:
+                self._client = SmartStreamClient(self._connect_to_server)
+            else:
+                self._client = client
         else:
             # credentials may be stripped from the base in some circumstances
             # as yet to be clearly defined or documented, so copy them.

